# test_housemate.py module

import profile
import unittest
from unittest.mock import patch, MagicMock

import sys
import io
import os
import pandas as pd
import random


from housemate.housemate import (
    main_menu,
    profile_menu,
    housemate_menu,
    rental_user_input,
    purchase_user_input,
    rental_main,
    purchase_main,
    rental_recommendation_main,
    get_file_path
)

from src.housemate_app.user.security import string_hash, reverse_hash, check_credentials
from src.housemate_app.user.userprofile import (
#    UserProfile,
    load_user_profiles,
    create_profile_from_input,
    append_to_dataframe,
    save_dataframe_to_csv
)
from src.housemate_app.user.userlogin import view_profile, edit_profile, delete_profile

from src.housemate_app.my_property.property import Property
from src.housemate_app.my_property.purchase import (
    Purchase,
    Condo,
    TownHome,
    Duplex,
    Bungalow,
    TwoStory,
    Mansion,
    gen_purchase,
    view_purchase_list,
    purchase_recommendation
)

from src.housemate_app.my_property.rental import (
    Rental,
    RentalCondo,
    RentalTownHome,
    RentalDuplex,
    RentalBungalow,
    RentalTwoStory,
    RentalMansion,
    gen_rental,
    view_rental_list,
    rental_recommendation
)


class TestMenus(unittest.TestCase):

    @ classmethod
    def classSetUp(clas):
        pass

    def setUp(self):
        self.main_output = (
            "Welcome to HouseMate! Please choose an option: \n"
            "1. Create Profile\n"
            "2. Login\n"
            "3. Exit HouseMate\n")
        self.profile_output = (
            "You are logged in. Please choose an option: \n"
            "1. View Profile Information\n"
            "2. Edit Profile\n"
            "3. Delete Profile\n"
            "4. View available homes\n"
            "5. Logout and return to the main menu\n"
        )
        self.housemate_output = (
            "Find a home today! Please choose an option: \n"
            "1. View ONLY recommended properties to rent\n"
            "2. View ONLY recommended properties to purchase\n"
            "3. View available rental properties\n"
            "4. View available purchase properties\n"
            "5. Return to the profile menu\n"
            "6. Logout and return to the main menu\n")

    @patch('builtins.input', side_effect=['1', '2', '3'])
    def test_main_menu_choices(self, mock_input):
        with patch('sys.stdout', new_callable=io.StringIO) as mock_stdout:
            main_menu()
            self.assertEqual(mock_stdout.getvalue(), self.main_output)

    @patch('builtins.input', side_effect=['1', '2', '3', '4', '5'])
    def test_profile_menu_choices(self, mock_input):
        with patch('sys.stdout', new_callable=io.StringIO) as mock_stdout:
            profile_menu()
            self.assertEqual(mock_stdout.getvalue(), self.profile_output)

    @patch('builtins.input', side_effect=['1', '2', '3', '4', '5', '6'])
    def test_housemate_menu_choices(self, mock_input):
        with patch('sys.stdout', new_callable=io.StringIO) as mock_stdout:
            housemate_menu()
            self.assertEqual(mock_stdout.getvalue(), self.housemate_output)
  
# housemate.py 21% to 24%, overall 75% to 76%                  
    # @patch('sys.stdout', new_callable=io.StringIO)
    # @patch('builtins.input', side_effect=['1', '2', '3'])  # Mimicking user input
    # def test_main_menu_choices(self, mock_input, mock_stdout):
    #     main_menu()
    #     expected_output = (
    #         "Welcome to HouseMate! Please choose an option: \n"
    #         "1. Create Profile\n"
    #         "2. Login\n"
    #         "3. Exit HouseMate\n"
    #     )
    #     self.assertEqual(mock_stdout.getvalue(), expected_output)

    # @patch('sys.stdout', new_callable=io.StringIO)
    # @patch('builtins.input', side_effect=['1', '2', '3', '4', '5'])  # Mimicking user input
    # def test_profile_menu_choices(self, mock_input, mock_stdout):
    #     profile_menu()
    #     expected_output = (
    #         "You are logged in. Please choose an option: \n"
    #         "1. View Profile Information\n"
    #         "2. Edit Profile\n"
    #         "3. Delete Profile\n"
    #         "4. View available homes\n"
    #         "5. Logout and return to the main menu\n"
    #     )
    #     self.assertEqual(mock_stdout.getvalue(), expected_output)

    # @patch('sys.stdout', new_callable=io.StringIO)
    # @patch('builtins.input', side_effect=['1', '2', '3', '4', '5', '6'])  # Mimicking user input
    # def test_housemate_menu_choices(self, mock_input, mock_stdout):
    #     housemate_menu()
    #     expected_output = (
    #         "Find a home today! Please choose an option: \n"
    #         "1. View ONLY recommendated properties to rent\n"
    #         "2. View ONLY recommendated properties to purchase\n"
    #         "3. View available rental properties\n"
    #         "4. View available purchase properties\n"
    #         "5. Return to the profile menu\n"
    #         "6. Logout and return to the main menu\n"
    #     )
    #     self.assertEqual(mock_stdout.getvalue(), expected_output)
    
    # ------------------------------------------------------------------------------------- also 21% to 24%, 75% to 76%
    #     def setUp(self):
    #     self.main_output = (
    #         "Welcome to HouseMate! Please choose an option: \n"
    #         "1. Create Profile\n"
    #         "2. Login\n"
    #         "3. Exit HouseMate\n")
    #     self.profile_output = (
    #         "You are logged in. Please choose an option: \n"
    #         "1. View Profile Information\n"
    #         "2. Edit Profile\n"
    #         "3. Delete Profile\n"
    #         "4. View available homes\n"
    #         "5. Logout and return to the main menu\n"
    #     )
    #     self.housemate_output = (
    #         "Find a home today! Please choose an option: \n"
    #         "1. View ONLY recommended properties to rent\n"
    #         "2. View ONLY recommended properties to purchase\n"
    #         "3. View available rental properties\n"
    #         "4. View available purchase properties\n"
    #         "5. Return to the profile menu\n"
    #         "6. Logout and return to the main menu\n")

    # def test_main_menu_choices(self):
    #     with patch('sys.stdout', new_callable=io.StringIO) as mock_stdout:
    #         main_menu()
    #         self.assertEqual(mock_stdout.getvalue(), self.main_output)

    # def test_profile_menu_choices(self):
    #     with patch('sys.stdout', new_callable=io.StringIO) as mock_stdout:
    #         profile_menu()
    #         self.assertEqual(mock_stdout.getvalue(), self.profile_output)

    # def test_housemate_menu_choices(self):
    #     with patch('sys.stdout', new_callable=io.StringIO) as mock_stdout:
    #         housemate_menu()
    #         self.assertEqual(mock_stdout.getvalue(), self.housemate_output)
            
# -----------------------------------------------------------------------------------------------------------------

    def tearDown(self):
        sys.stdout = sys.__stdout__

    @classmethod
    def tearDownClass(cls):
        # tearDownClass is a requirement
        pass


class TestHelperFunctions(unittest.TestCase):
    @ classmethod
    def classSetUp(clas):
        pass

    def setUp(self):
        random.seed(123)

    def test_get_file_path(self):
        self.assertEqual(get_file_path(), os.getcwd()+"\\user_profiles.csv")

    @patch('builtins.input', return_value='tEsT')
    def test_rental_user_input(self, mock_input):
        self.assertEqual(rental_user_input(), 'test')

    @patch('builtins.input', return_value='ABCD')
    def test_purchase_user_input(self, mock_input):
        self.assertEqual(purchase_user_input(), 'abcd')

    def tearDown(self):
        sys.stdout = sys.__stdout__

    @classmethod
    def tearDownClass(cls):
        # tearDownClass is a requirement
        pass


if __name__ == "__main__":
    unittest.main(argv=[''], verbosity=2, exit=False)


# test_userlogin.py

import os
import unittest
from unittest.mock import patch
from io import StringIO
import pandas as pd

from src.housemate_app.user.userlogin import login_get_file_path, view_profile, edit_profile, delete_profile
from src.housemate_app.user.userprofile import load_user_profiles
from src.housemate_app.user.security import string_hash

class TestUserProfileLoading(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        # Create a common temporary CSV file for all test cases in the class
        cls.test_csv_filename = 'user_profiles.csv'

    def setUp(self):
        # Create a temporary CSV file for each test method with the expected filename
        with open(self.test_csv_filename, 'w') as file:
            file.write('User1,25,user1@email.com,username1,password1\n')
            file.write('User2,30,user2@email.com,username2,password2\n')
            file.write('User3,28,user3@email.com,username3,password3\n')

    def test_login_get_file_path(self):
        # Test the login_get_file_path function
        expected_path = os.path.abspath(self.test_csv_filename)
        file_path = login_get_file_path()
        self.assertEqual(file_path, expected_path)

    def test_load_user_profiles(self):
        # Test the load_user_profiles function
        file_path = os.path.abspath(self.test_csv_filename)
        user_profiles = load_user_profiles(file_path)

        # Ensure user_profiles is a DataFrame and has data rows
        self.assertIsInstance(user_profiles, pd.DataFrame)
        self.assertGreater(len(user_profiles), 0)
        
        # Additional assertions for DataFrame structure
        # Check column names
        self.assertListEqual(list(user_profiles.columns), ['User1', '25', 'user1@email.com', 'username1', 'password1'])
        # Check number of columns
        self.assertEqual(len(user_profiles.columns), 5)
        # Check number of rows
        self.assertEqual(len(user_profiles.index), 2)

    def tearDown(self):
        # Remove the temporary CSV file created for each test method
        os.remove(self.test_csv_filename)

    @classmethod
    def tearDownClass(cls):
        # Clean up the common temporary CSV file created for all test cases in the class
        if os.path.exists(cls.test_csv_filename):
            os.remove(cls.test_csv_filename)

        
class TestViewProfile(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        # Create a common temporary CSV file for all test cases in the class
        cls.test_csv_filename = 'user_profiles.csv'

    def setUp(self):
        # Create a temporary CSV file for each test method with the expected filename
        sample_data = {
            'name': ['User1', 'User2', 'User3'],
            'age': [25, 30, 28],
            'email': ['user1@email.com', 'user2@email.com', 'user3@email.com'],
            'username': ['username1', 'username2', 'username3'],
            'password': ['password1', 'password2', 'password3']
        }
        self.df = pd.DataFrame(sample_data)
        self.df.to_csv(self.test_csv_filename, index=False)

    def test_view_profile_existing_user(self):
        # Test the view_profile function
        # Simulate user's profile information
        username_hash = string_hash('username1')
        password_hash = string_hash('password1')
        
        user_profile_data = {
            'name': ['User1'],
            'age': [25],
            'email': ['user1@email.com'],
            'username': [username_hash],  # Hashed value for username to match
            'password': [password_hash]   # Hashed value for password to match
        }
        user_df = pd.DataFrame(user_profile_data)

        with unittest.mock.patch('sys.stdout', new=StringIO()) as fake_output:
            view_profile('username1', user_df)
            captured_output = fake_output.getvalue().strip()

        # Check if the output contains profile information
        if 'Profile Information' in captured_output:
            expected_output = (
                "Profile Information:\n    name  age            email  username  password\n"
                "0  User1   25  user1@email.com      3337      3479"
            ).strip()
            self.assertEqual(expected_output, captured_output)
        else:
            # If the output doesn't contain profile information, it should contain the 'No profile information found' message
            expected_output = "No profile information found for this user."
            self.assertEqual(expected_output, captured_output)

        # Checking if the output contains 'Profile Information'
        self.assertIn('Profile Information', captured_output)

        # Ensure the output format matches the expected structure
        self.assertTrue(captured_output.startswith("Profile Information:"))
        self.assertTrue(captured_output.endswith("3479"))

        # Checking the length of the captured output
        self.assertGreater(len(captured_output), 0)

        # Checking specific values in the output
        self.assertIn('User1', captured_output)
        self.assertIn('25', captured_output)
        self.assertIn('user1@email.com', captured_output)
        self.assertIn('3337', captured_output)
        self.assertIn('3479', captured_output)

    def test_view_profile_nonexistent_user(self):
        # Test the view_profile function for users that don't exist
        expected_output = "No profile information found for this user.\n"

        with unittest.mock.patch('sys.stdout', new=StringIO()) as fake_output:
            view_profile('nonexistent_user', self.df)
            captured_output = fake_output.getvalue()

        self.assertEqual(expected_output, captured_output)

        # Checking if the output contains 'No profile information found'
        self.assertIn('No profile information found', captured_output)

        # Checking the length of the captured output
        self.assertEqual(len(captured_output), len(expected_output))

        # Checking for the absence of specific values
        self.assertNotIn('User1', captured_output)
        self.assertNotIn('25', captured_output)
        self.assertNotIn('user1@email.com', captured_output)
        self.assertNotIn('3337', captured_output)
        self.assertNotIn('3479', captured_output)

class TestEditProfile(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        # Create a temporary dataframe for each test method with the expected output
        cls.sample_data = {
            'name': ['User1'],
            'age': [25],
            'email': ['new_email@email.com'],
            'username': ['username1'],
            'password': ['password1']
        }
        cls.df = pd.DataFrame(cls.sample_data)

    def setUp(self):
        pass

    @patch('builtins.input', side_effect=['email', 'new_email@email.com'])
    def test_edit_profile_existing_field(self, mock_input):
        # Test the edit_profile function
        expected_output = "Profile updated successfully!\n"
        result = edit_profile('User1', self.df)
        self.assertEqual(result, ("no_change", None))
        # Checking if the email field is updated in the dataframe
        self.assertEqual(self.df.loc[0, 'email'], 'new_email@email.com')

    @patch('builtins.input', side_effect=['invalid_field'])
    def test_edit_profile_invalid_field(self, mock_input):
        expected_output = "Invalid field name. Profile not updated.\n"
        result = edit_profile('User1', self.df)
        # Checking if there is an invalid input
        self.assertEqual(result, ("no_change", None))
        # Checking if the profile remains unchanged
        self.assertEqual(self.df.loc[0, 'email'], 'new_email@email.com')

    def tearDown(self):
        pass

    @classmethod
    def tearDownClass(cls):
        # Cleanup or reset any values that were modified in setUpClass
        cls.sample_data = None
        cls.df = None
        
class TestDeleteProfile(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        # Create a temporary CSV file for each test method with the expected filename
        cls.test_csv_filename = 'user_profiles.csv'
        cls.sample_data = {
            'name': ['User1', 'User2', 'User3'],
            'age': [25, 30, 28],
            'email': ['user1@email.com', 'user2@email.com', 'user3@email.com'],
            'username': ['username1', 'username2', 'username3'],
            'password': ['password1', 'password2', 'password3']
        }
        cls.df = pd.DataFrame(cls.sample_data)
        cls.df.to_csv(cls.test_csv_filename, index=False)

    def setUp(self):
        pass

    @patch('builtins.input', return_value='yes')
    def test_delete_profile_confirmation_yes(self, mock_input):
        # Test the delete_profile function with 'yes' confirmation
        result = delete_profile('username1', self.df)
        self.assertEqual(result, 3337)

    @patch('builtins.input', return_value='no')
    def test_delete_profile_confirmation_no(self, mock_input):
        # Test the delete_profile function with 'no' confirmation
        result = delete_profile('username1', self.df)
        self.assertEqual(result, 3337)
    
    @patch('builtins.input', side_effect=['yes'])
    def test_delete_profile_decorator(self, mock_input):
        # Simulate user's profile information
        username_hash = string_hash('username1')
        password_hash = string_hash('password1')
        
        user_profile_data = {
            'name': ['User1'],
            'age': [25],
            'email': ['user1@email.com'],
            'username': [username_hash],  # Hashed value for username to match
            'password': [password_hash]   # Hashed value for password to match
        }

        # Create a DataFrame with the user profile data
        user_df = pd.DataFrame(user_profile_data)

        # Simulate the function call and capture the output
        with unittest.mock.patch('sys.stdout', new=StringIO()) as fake_output:
            delete_profile('username1', user_df)  # Pass the original username for testing
            captured_output = fake_output.getvalue().strip()

        # Ensure the profile is deleted from the DataFrame
        self.assertEqual(len(user_df), 0)  # Assuming profile deletion means an empty DataFrame

        # Check the captured output message
        expected_output = "Profile deleted successfully!"
        self.assertEqual(expected_output, captured_output)

    def tearDown(self):
        # Remove the temporary CSV file after each test
        if os.path.exists(self.test_csv_filename):
            os.remove(self.test_csv_filename)

    @classmethod
    def tearDownClass(cls):
        # Cleanup or reset any values that were modified in setUpClass
        cls.test_csv_filename = None
        cls.sample_data = None
        cls.df = None

if __name__ == '__main__':
    unittest.main(argv=[''], verbosity=2, exit=False)





    # housemate_main.py module

if __name__ == "__main__":
    import os
    import pandas as pd

    from housemate_app.user.security import string_hash, reverse_hash, check_credentials
    from housemate_app.user.userprofile import (
    #    UserProfile,
        load_user_profiles,
        create_profile_from_input,
        append_to_dataframe,
        save_dataframe_to_csv
    )
    from housemate_app.user.userlogin import view_profile, edit_profile, delete_profile

    from housemate_app.my_property.property import Property
    from housemate_app.my_property.purchase import (
        Purchase, 
        Condo,
        TownHome,
        Duplex,
        Bungalow,
        TwoStory,
        Mansion,
        gen_purchase, 
        view_purchase_list,
        purchase_recommendation
    )
    from housemate_app.my_property.rental import (
        Rental,
        RentalCondo,
        RentalTownHome,
        RentalDuplex,
        RentalBungalow,
        RentalTwoStory,
        RentalMansion,
        gen_rental,
        view_rental_list,
        rental_recommendation
    )

    # --------------------------------- functions start here --------------------------------- #

    # Function to establish file_path and load user profiles from CSV file using load_user_profiles from userprofile.py module
    def get_file_path():
        # Get the current working directory and set the file path
        current_directory = os.getcwd()
        file_path = os.path.join(current_directory, 'user_profiles.csv')
        return file_path

    file_path = get_file_path()
    user_profiles = load_user_profiles(file_path)

    # Function to display main menu options
    def main_menu():
        print("Welcome to HouseMate! Please choose an option: ")
        print("1. Create Profile")
        print("2. Login")
        print("3. Exit HouseMate")

    # Function to display profile menu options
    def profile_menu():
        print("You are logged in. Please choose an option: ")
        print("1. View Profile Information")
        print("2. Edit Profile")
        print("3. Delete Profile")
        print("4. View available homes")
        print("5. Logout and return to the main menu")
        
    # Function to display the housemate menu options
    def housemate_menu(): 
        print("Find a home today! Please choose an option: ")
        print("1. View ONLY recommendated properties to rent")
        print("2. View ONLY recommendated properties to purchase")
        print("3. View available rental properties")
        print("4. View available purchase properties")
        print("5. Return to the profile menu")
        print("6. Logout and return to the main menu")
        
    # Function to obtain the user input and assign to a variable for use in casting onto a class
    def rental_user_input():
        print("Enter the preferred type of property (Condo/TownHome/Duplex/Bungalow/TwoStory/Mansion): ")
        user_input = input().lower()
        return user_input

    # Function to obtain the mapped user input using rental_user_input() above, to a variable to be called into a function from rental.py
    def rental_main():
        while True:
            try:
                rent_n = int(input("Enter the number of rental properties you would like to view (1-9): "))
                if 1 <= rent_n <= 9:
                    while True:
                        user_pref = rental_user_input()

                        if user_pref == 'condo': # Ensure RentalCondo, RentalTownHome, etc., are the class names, not instances
                            pref_type = RentalCondo
                        elif user_pref == 'townhome':
                            pref_type = RentalTownHome
                        elif user_pref == 'duplex':
                            pref_type = RentalDuplex
                        elif user_pref == 'bungalow':
                            pref_type = RentalBungalow
                        elif user_pref == 'twostory':
                            pref_type = RentalTwoStory
                        elif user_pref == 'mansion':
                            pref_type = RentalMansion
                        else:
                            print("Invalid property type entered. Please try again.")
                            continue

                        rental_list = gen_rental(rent_n, pref_type)
                        view_rental_list(rental_list) # Display 'n' available rental properties
                        break
                    break
                else:
                    print("The number of properties should be between 1 and 9.")
            except ValueError:
                print("Please enter only whole numbers.")

    # Function to obtain the user input and assign to a variable for use in casting onto a class 
    def purchase_user_input():
        print("Enter the preferred type of property (Condo/TownHome/Duplex/Bungalow/TwoStory/Mansion): ")
        user_input = input().lower()
        return user_input

    # Function to obtain the mapped user input using purchase_user_input() above, to a variable to be called into a function from purchase.py
    def purchase_main():
        while True:
            try: 
                purchase_n = int(input("Enter the number of purchase properties you would like to view (1-9): "))
                if 1 <= purchase_n <= 9:
                    while True:
                        user_pref = purchase_user_input()
                        
                        if user_pref == 'condo': # Ensure Condo, TownHome, etc., are the class names, not instances
                            pref_type = Condo
                        elif user_pref == 'townhome':
                            pref_type = TownHome
                        elif user_pref == 'duplex':
                            pref_type = Duplex
                        elif user_pref == 'bungalow':
                            pref_type = Bungalow
                        elif user_pref == 'twostory':
                            pref_type = TwoStory
                        elif user_pref == 'mansion':
                            pref_type = Mansion
                        else:
                            print("Invalid property type entered. Please try again.")
                            continue

                        purchase_list = gen_purchase(purchase_n, pref_type)
                        view_purchase_list(purchase_list) # Display 'n' available purchase properties
                        break
                    break 
                else:
                    print("The number of properties should be between 1 and 9.")
            except ValueError:
                print("Please enter only whole numbers.")
                
    # Function to obtain the mapped user input using rental_user_input() above, to a variable to be called into a function from rental.py
    def rental_recommendation_main():
        while True:
            try: 
                income = int(input("Enter your annual salary: "))
                while not (1 <= income <= 10000000):
                    print("Please enter an annual salary between 1 and $10,000,000")
                    income = int(input("Enter your annual salary: "))

                pref_beds = int(input("Enter the number of bedrooms you would prefer: "))
                while not (1 <= pref_beds <= 10):
                    print("Please enter a preferred number of bedrooms between 1 and 10")
                    pref_beds = int(input("Enter the number of bedrooms you would prefer: "))

                prop_type = rental_user_input()
                prop_type_mapping = {
                    'condo': RentalCondo, # Ensure RentalCondo, RentalTownHome, etc., are the class names, not instances
                    'townhome': RentalTownHome,
                    'duplex': RentalDuplex,
                    'bungalow': RentalDuplex,
                    'twostory': RentalTwoStory,
                    'mansion': RentalMansion
                }

                rec_rent_list = []
                if prop_type in prop_type_mapping:
                    prop_class = prop_type_mapping[prop_type]  # Retrieve class from the mapping
                    rec_rent_list = view_rental_list(rental_recommendation(prop_class, income, pref_beds))
                    # Display rental recommendations
                    break
                else:
                    print("Invalid property type entered. Please try again.")

                break # Exiting the input loop once all inputs are taken

            except ValueError:
                print("Please enter only whole numbers.")

    # Function use the map user input using purchase_user_input() above, to an variable to be called into a function from purchase.py         
    def purchase_recommendation_main():
        while True:
            try: 
                downpay = int(input("Enter the amount you have for a downpayment: "))
                while not (1 <= downpay <= 10000000):
                    print("Please enter a downpayment between 1 and $10,000,000")
                    downpay = int(input("Enter the amount you have for a downpayment: "))

                income = int(input("Enter your annual salary: "))
                while not (1 <= income <= 10000000):
                    print("Please enter an annual salary between 1 and $10,000,000")
                    income = int(input("Enter your annual salary: "))

                pref_beds = int(input("Enter the number of bedrooms you would prefer: "))
                while not (1 <= pref_beds <= 10):
                    print("Please enter a preferred number of bedrooms between 1 and 10")
                    pref_beds = int(input("Enter the number of bedrooms you would prefer: "))

                prop_type = purchase_user_input()
                prop_type_mapping = {
                    'condo': Condo, # Ensure Condo, TownHome, etc., are the class names, not instances
                    'townhome': TownHome,
                    'duplex': Duplex,
                    'bungalow': Bungalow,
                    'twostory': TwoStory,
                    'mansion': Mansion
                }

                rec_pur_list = []
                if prop_type in prop_type_mapping:
                    prop_class = prop_type_mapping[prop_type]  # Retrieve class from the mapping
                    rec_pur_list = view_purchase_list(purchase_recommendation(prop_class, downpay, income, pref_beds))
                    # Display purchase recommendations
                    break

                else:
                    print("Invalid property type entered. Please try again.")

                break # Exiting the input loop once all inputs are taken

            except ValueError:
                print("Please enter only whole numbers.")

    # --------------------------------- functions end here --------------------------------- #

    class MaximumProfileLimitReached(Exception):
        """Custom exception for when the maximum profile limit is reached."""

        def __init__(self, message="Maximum profile limit reached (3 profiles created)."):
            self.message = message
            super().__init__(self.message)  # Properly initialize the Exception class
            # print(self.message)

    # Flag to track successful login
    logged_in = False

    status = True

    while status == True:
        main_menu_active = True  # Setting main menu flag to true for flow control
        while main_menu_active == True:
            main_menu()
            choice = input("Enter your choice (1-3): ")

            # Load user profiles from CSV file using load_user_profiles from userprofile.py module
            user_profiles = load_user_profiles(file_path)

            if choice == '1':
                # Conditional if statement to ensure that the create profile function isn't implicitly called in userlogin.py module
                if __name__ == "__main__":
                    file_name = 'user_profiles.csv'

                    existing_df = load_user_profiles(file_path)

                    # Conditional if statement that checks if there is an existing dataframe, if not create one
                    if existing_df is None:
                        existing_df = pd.DataFrame()

                    profile_count = 0  # Variable to track the number of profiles created

                    while profile_count < 3:  # Limit creation of profiles to a maximum of 3
                        user_profile_data = create_profile_from_input()
                        existing_df = append_to_dataframe(existing_df, user_profile_data)
                        profile_count += 1  # Increment profile count after creating a profile

                        remaining_profiles = 3 - profile_count
                        if remaining_profiles > 0:
                            print(f"You have {remaining_profiles} profile creations remaining.")
                        else:
                            try:
                                raise MaximumProfileLimitReached()  # Use the custom exception to print the message and skip further execution
                            except MaximumProfileLimitReached as e:
                                print(e)
                                break  # Exit the loop when the maximum profile limit is reached

                        # Ask if the user wants to add another profile
                        choice = input("Do you want to add another profile? (yes/no): ").lower()
                        while choice not in ['yes', 'no']:  # Loop until valid input is received
                            print("Please enter either yes or no.")
                            choice = input("Do you want to add another profile? (yes/no): ").lower()

                        if choice == 'no':
                            break  # Exit profile creation loop if the user chooses 'no'

                    main_menu_active = True  # Setting main menu flag to false for flow control (sent back to main menu to login)

            elif choice == '2':
                # User login to get to the profile menu
                # While loop to take in the username and password and match to credentials stored in CSV, if no match continues asking
                
                # User is allowed 3 attempts before being exited
                max_attempts = 3
                attempts = 0

                while attempts < max_attempts:
                    username = input("Enter your username or 's' to stop: ")
                    if username == 's':
                        break
                    password = input("Enter your password or 's' to stop: ")
                    if password == 's':
                        break
                    
                    if check_credentials(username, password, user_profiles):
                        print("Login Successful!")
                        logged_in = True # Set login to true to access profile menu
                        break
                    else:
                        attempts += 1
                        remaining_attempts = max_attempts - attempts
                        if remaining_attempts > 0:
                            print(f"Invalid credentials. {remaining_attempts} attempts remaining.")
                        else:
                            print("Maximum attempts reached. Exiting.")
                            break
                main_menu_active = False # Setting main menu flag to false for flow control (sent to the next menu)

            elif choice == '3':
                print("Exiting HouseMate. Have a great day!")
                status = False
                main_menu_active = False # Setting main menu flag to false for flow control (exits from application)
            else:
                print("Invalid choice. Please enter a number between 1 and 3 or 'q' to exit HouseMate.")

        if status == False:
            break

        # Check login status before entering the profile menu
        if logged_in:
            profile_menu_active = True # Setting the profile menu flag to true for flow control
            while profile_menu_active == True:
                # Profile menu after successful login
                profile_menu()
                choice = input("Enter your choice (1-5) or 'q' to exit HouseMate: ")

                if choice == '1':
                    # View Profile Information
                    view_profile(username, user_profiles)
                elif choice == '2':
                    # Edit Profile
                    edit_result, new_value = edit_profile(username, user_profiles)
                    if edit_result == "username_changed":
                        print("Username has been changed. Please login again.")
                        break # Exit the profile menu loop when the username or password is edited (to re login)
                    elif edit_result == "password_changed":
                        print("Password has been changed. Please login again.")
                        break # Exit the profile menu loop when the username or password is edited (to re login)
                elif choice == '3':
                    deleted_user = delete_profile(username, user_profiles) # Utilize the decorated function
                    if string_hash(username) == deleted_user:
                        print("Profile has been deleted. Please log in again.")
                    break  # Exit the profile menu loop when the user's profile is deleted (sent to the main menu)
                elif choice == '4':
                    housemate_menu_active = True # Setting housemate menu flag to true for flow control
                    while housemate_menu_active == True:
                        # Housemate menu for selecting recommendations or viewing all
                        housemate_menu()
                        choice = input("Enter your choice (1-5) or 'q' to exit HouseMate: ")

                        if choice == '1':
                            # Rental recommendation list based on user input
                            print("This is showing you the recommended properties to rent")
                            rental_recommendation_main()
                            continue                
                        if choice == '2':
                            # Purchase recommendation list based on user input
                            print("This is showing you the recommended properties to purchase")
                            purchase_recommendation_main()
                            continue        
                        elif choice == '3':
                            # Find a home to rent
                            print("This is finding you a home to rent")
                            if __name__ == "__main__":
                                rental_main()
                            continue
                        elif choice == '4':
                            # Find a home to purchase
                            print("This is finding you a home to purchase")
                            if __name__ == "__main__":
                                purchase_main()
                            continue
                        elif choice == '5':
                            # Return to the main menu
                            print("This is returning to the profile menu")
                            housemate_menu_active = False # Setting housemate menu flag to false for flow control (sent to profile menu)
                        elif choice == '6':
                            # Logout and return to the main menu
                            print("This is returning to the main menu")
                            housemate_menu_active = False # Setting housemate menu flag to false for flow control (sent to profile menu)
                            profile_menu_active = False # Setting profile menu flag to false for flow control (sent to the main menu)
                        elif choice.lower() == 'q':
                            # Exit HouseMate when 'q' is entered (exits application)
                            print("Exiting HouseMate. Have a great day!")
                            status = False
                            housemate_menu_active = False # Setting housemate menu flag to false for flow control
                            profile_menu_active = False # Setting profile menu flag to false for flow control
                            main_menu_active = False # Setting main menu flag to false for flow control
                        else:
                            print("Invalid choice. Please enter a number between 1 and 5 or 'q' to exit HouseMate.")
                elif choice == '5':
                    # Logout and return to the main menu
                    print("This is returning to the main menu.")
                    break  # Exit the menu loop when logging out
                elif choice.lower() == 'q':
                    print("Exiting HouseMate. Have a great day!")
                    status = False
                    break  # Exit HouseMate when 'q' is entered (exits application)
                else:
                    print("Invalid choice. Please enter a number between 1 and 5 or 'q' to exit HouseMate.")
            
            if status == False:
                break # Exit the outer while loop when the user chooses to exit HouseMate
        
        



# test_userprofile.py module

import os
import io
import unittest
from unittest.mock import patch
import pandas as pd

from src.housemate_app.user.userprofile import load_user_profiles, create_profile_from_input, append_to_dataframe, save_dataframe_to_csv
from src.housemate_app.user.security import string_hash

class TestUserProfile(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        pass

    def setUp(self):
        # Create a temporary CSV file for testing
        self.test_csv_filename = 'test_user_profiles.csv'
        self.test_file_path = os.path.join(os.getcwd(), self.test_csv_filename)
        self.sample_data = {
            'name': ['Test User'],
            'age': [25],
            'email': ['test_user@example.com'],
            'username': ['hashed_test_user'],
            'password': ['hashed_password']
        }
        self.df = pd.DataFrame(self.sample_data)
        self.df.to_csv(self.test_file_path, index=False)

    def test_create_profile_from_input(self):
        # Simulate creating a user profile from user input
        profile_data = {
            'name': 'User1',
            'age': 25,
            'email': 'user1@example.com',
            'username': 'username1',  # Keeping the original username and password
            'password': 'password1'
        }

        # Mock the user input process
        def mock_input(prompt):
            if 'Enter your name: ' in prompt:
                return profile_data['name']
            elif 'Enter your age: ' in prompt:
                return str(profile_data['age'])
            elif 'Enter your email: ' in prompt:
                return profile_data['email']
            elif 'Enter your username (8 characters or more): ' in prompt:
                return profile_data['username']
            elif 'Enter your password (8 characters or more): ' in prompt:
                return profile_data['password']

        with unittest.mock.patch('builtins.input', side_effect=mock_input):
            created_profile_data = create_profile_from_input()

        # Hash the username and password before assertion
        hashed_username = string_hash(profile_data['username'])
        hashed_password = string_hash(profile_data['password'])

        # Replace the original username and password with hashed values
        profile_data['username'] = hashed_username
        profile_data['password'] = hashed_password

        # Verify if the created profile matches the expected data
        self.assertEqual(created_profile_data, profile_data)
        self.assertEqual(len(created_profile_data), 5)
        self.assertEqual(created_profile_data['age'], 25)
        self.assertIn('user1@example.com', created_profile_data.values())

    def tearDown(self):
        # Remove the temporary CSV file after each test
        if os.path.exists(self.test_file_path):
            os.remove(self.test_file_path)

    @classmethod
    def tearDownClass(cls):
        pass

class TestAppendToDataFrame(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        # Set up initial data for testing append_to_dataframe function
        cls.initial_data = {
            'name': ['User1'],
            'age': [25],
            'email': ['user1@example.com'],
            'username': ['username1'],
            'password': ['password1']
        }
        cls.initial_df = pd.DataFrame(cls.initial_data)

    def setUp(self):
        pass

    def test_append_to_dataframe(self):
        # Append a new profile to the existing dataframe
        new_profile_data = {
            'name': 'User2',
            'age': 30,
            'email': 'user2@example.com',
            'username': 'username2',
            'password': 'password2'
        }
        new_df = append_to_dataframe(self.initial_df, new_profile_data)

        # Verify if the new dataframe contains the added profile
        self.assertEqual(len(new_df), len(self.initial_df) + 1)
        self.assertListEqual(list(new_df.columns), list(self.initial_df.columns))
        self.assertIn('username2', new_df['username'].values)
        self.assertEqual(new_df.loc[new_df['username'] == 'username2', 'age'].values[0], 30)

    def tearDown(self):
        pass

    @classmethod
    def tearDownClass(cls):
        pass

class TestSaveDataFrameToCSV(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        # Set up initial data for testing save_dataframe_to_csv function
        cls.test_data = {
            'name': ['User1'],
            'age': [25],
            'email': ['user1@example.com'],
            'username': ['username1'],
            'password': ['password1']
        }
        cls.test_df = pd.DataFrame(cls.test_data)
        cls.test_file_path = os.path.join(os.getcwd(), 'test_save.csv')
        
    def setUp(self):
        pass

    def test_save_dataframe_to_csv(self):
        # Save the dataframe to a temporary CSV file
        save_dataframe_to_csv(self.test_df, self.test_file_path)

        # Check if the file was created and contains data
        self.assertTrue(os.path.exists(self.test_file_path))
        loaded_df = load_user_profiles(self.test_file_path)
        self.assertIsNotNone(loaded_df)
        self.assertEqual(len(loaded_df), len(self.test_df))
        self.assertListEqual(list(loaded_df.columns), list(self.test_df.columns))
        
    def tearDown(self):
        pass
    
    @classmethod
    def tearDownClass(cls):
        # Clean up class-level resources
        if os.path.exists(cls.test_file_path):
            os.remove(cls.test_file_path)

if __name__ == '__main__':
    unittest.main(argv=[''], verbosity=2, exit=False)





# property.py module

import random

# Superclass Property
class Property(object):
    def __init__(self, sqft, num_beds, num_baths):
        self.sqft = sqft
        self.num_beds = num_beds
        self.num_baths = num_baths

    def display_property(self):
        print(f'The price of this property is: ${self.price}.00')
        print(f'The square footage of this property is: {self.sqft} ft.')
        print(f'This property has {self.num_beds} bedrooms')
        print(f'This property has {self.num_baths} bathrooms')


# prop_list = []


# def gen_n_properties(n):
#     for i in range(n):
#         prop_list.append(Property())
#         print(f'Property {i} created')


# Note: p1 = Person() # autofills all attributes thanks to second __init__

# example: gen_n_properties(3)
# output: Property 0 created
# Property 1 created
# Property 2 created
# Now we can reference prop_list[i].display_property


# Problems:
# It generates truly random properties--
# Meaning it can give a house for $300,000 and 44999 sqft, but only 2 bed and 1 bath
# Secondly, price doesn't belong in Property (it belongs in Purchase) --
# I realized that once it was already in, but it's an easy copy + paste
# More to come!



# test_rental.py module

import unittest
import sys
import io
import random

from src.housemate_app.my_property.property import Property
from src.housemate_app.my_property.rental import (
    Rental,
    RentalCondo,
    RentalTownHome,
    RentalDuplex,
    RentalBungalow,
    RentalTwoStory,
    RentalMansion,
    gen_rental,
    rental_recommendation
)


class TestRental(unittest.TestCase):
    @ classmethod
    def setUpClass(cls):
        pass

    def setUp(self):
        random.seed(355)

        # Initializing test instances
        self.r_condo_inst = RentalCondo()
        self.r_townhome_inst = RentalTownHome()
        self.r_duplex_inst = RentalDuplex()
        self.r_bungalow_inst = RentalBungalow()
        self.r_twostory = RentalTwoStory()
        self.r_mansion = RentalMansion()
        self.rental_known = Rental(1504, 4, 3, 3500, 314)
        self.rental2_known = Rental(799, 1, 2, 1800, 121)
        self.rental3_known = Rental(3114, 5, 5, 6000, 446)
        self.rental_gen_result = gen_rental(3, RentalBungalow)

        # Saving original stdout
        # Redirect stdout to a StringIO object
        self.original_stdout = sys.stdout
        sys.stdout = io.StringIO()

    def tearDown(self):
        sys.stdout = self.original_stdout

    @classmethod
    def tearDownClass(cls):
        pass

    def test_rental_condo(self):
        self.assertTrue(350 <= self.r_condo_inst.sqft <= 800)
        self.assertTrue(1 <= self.r_condo_inst.num_beds <= 2)
        self.assertTrue(1 == self.r_condo_inst.num_baths)
        self.assertTrue(1400 <= self.r_condo_inst.rent <= 1900)
        self.assertTrue(100 <= self.r_condo_inst.utilities <= 150)

    def test_rental_townhome(self):
        self.assertTrue(800 <= self.r_townhome_inst.sqft <= 1800)
        self.assertTrue(2 <= self.r_townhome_inst.num_beds <= 4)
        self.assertTrue(1 <= self.r_townhome_inst.num_baths <= 3)
        self.assertTrue(2200 <= self.r_townhome_inst.rent <= 3200)
        self.assertTrue(200 <= self.r_townhome_inst.utilities <= 250)

    def test_rental_duplex(self):
        self.assertTrue(1400 <= self.r_duplex_inst.sqft <= 2000)
        self.assertTrue(2 <= self.r_duplex_inst.num_beds <= 5)
        self.assertTrue(2 <= self.r_duplex_inst.num_baths <= 4)
        self.assertTrue(2800 <= self.r_duplex_inst.rent <= 4000)
        self.assertTrue(250 <= self.r_duplex_inst.utilities <= 300)

    def test_bungalow(self):
        self.assertTrue(1300 <= self.r_bungalow_inst.sqft <= 2400)
        self.assertTrue(4 <= self.r_bungalow_inst.num_beds <= 5)
        self.assertTrue(3 <= self.r_bungalow_inst.num_baths <= 4)
        self.assertTrue(3400 <= self.r_bungalow_inst.rent <= 4600)
        self.assertTrue(300 <= self.r_bungalow_inst.utilities <= 400)

    def test_twostory(self):
        self.assertTrue(2000 <= self.r_twostory.sqft <= 3300)
        self.assertTrue(4 <= self.r_twostory.num_beds <= 6)
        self.assertTrue(3 <= self.r_twostory.num_baths <= 5)
        self.assertTrue(4500 <= self.r_twostory.rent <= 6100)
        self.assertTrue(350 <= self.r_twostory.utilities <= 450)

    def test_mansion(self):
        self.assertTrue(8000 <= self.r_mansion.sqft <= 12000)
        self.assertTrue(8 <= self.r_mansion.num_beds <= 10)
        self.assertTrue(7 <= self.r_mansion.num_baths <= 8)
        self.assertTrue(10000 <= self.r_mansion.rent <= 13000)
        self.assertTrue(500 <= self.r_mansion.utilities <= 675)

    def test_calc_total_rent(self):
        self.assertEqual(self.rental_known.calc_total_rent(), 45768)
        self.assertEqual(self.rental2_known.calc_total_rent(), 23052)

    def test_display_rental(self):
        # Below code specifies that the output is a print statement!
        to_terminal = io.StringIO()
        sys.stdout = to_terminal
        self.rental3_known.display_rental()

        expected_print = (
            "Rental type: Rental\n"
            "Monthly rent: $6000\n"
            "Monthly utilities: $446\n"
            "Pet deposit required: False\n"
            "Annual total: $77352\n"
            "Square footage: 3114sqft.\n"
            "Number of bedrooms: 5\n"
            "Number of bathrooms: 5"
        )
        self.assertEqual(to_terminal.getvalue().strip(), expected_print)

    def test_gen_rental(self):
        self.assertEqual(len(self.rental_gen_result), 3)
        for instance in self.rental_gen_result:
            self.assertIsInstance(instance, RentalBungalow)
        self.assertEqual(self.rental_gen_result[0].rent, 3468)
        self.assertEqual(self.rental_gen_result[1].sqft, 1896)
        self.assertEqual(self.rental_gen_result[2].num_beds, 4)

    def test_rental_recommendation(self):
        # Instantiating rental_recommendation here, in efforts to suppress the print messages
        self.rental_recommendation_result = rental_recommendation(
            RentalCondo, 37000, 1)

        self.assertEqual(len(self.rental_recommendation_result), 15)
        for instance in self.rental_recommendation_result:
            self.assertIsInstance(instance, RentalCondo)
        self.assertEqual(self.rental_recommendation_result[10].rent, 1677)
        self.assertEqual(self.rental_recommendation_result[7].sqft, 844)
        self.assertEqual(self.rental_recommendation_result[2].num_baths, 1)


if __name__ == '__main__':
    unittest.main(argv=[''], verbosity=2, exit=False)



# test_security.py module

import unittest
import pandas as pd

from src.housemate_app.user.security import string_hash, reverse_hash, check_credentials

class TestStringHash(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.mapped_strings = {}

    def setUp(self):
        pass

    def test_string_hash_basic(self):
        # Test for basic string hashing
        result = string_hash("This is a test to see what happens")
        self.assertEqual(result, 53286)
        self.assertNotEqual(result, 0)
        self.assertIsInstance(result, int)
        self.assertGreater(result, 0)

    def test_string_hash_sensitive_info(self):
        # Test for hashing sensitive information
        result = string_hash("SensitiveInformationIsHere")
        self.assertEqual(result, 33376)
        self.assertNotEqual(result, 0)
        self.assertIsInstance(result, int)
        self.assertGreater(result, 0)

    def test_string_hash_special_characters(self):
        # Test for hashing strings with numbers and symbols
        result = string_hash("What if there are numbers (333!) and symbols too?")
        self.assertEqual(result, 101197)
        self.assertNotEqual(result, 0)
        self.assertIsInstance(result, int)
        self.assertGreater(result, 0)
        
    def tearDown(self):
        pass

    @classmethod
    def tearDownClass(cls):
        cls.mapped_strings = None
class TestReverseHash(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.mapped_strings = {}

    def setUp(self):
        pass

    def test_reverse_hash_basic(self):
        # Test for reversing hashed values
        string_hashed = "This is a test to see what happens"
        hash_value = string_hash(string_hashed)
        self.mapped_strings[hash_value] = string_hashed

        result = reverse_hash(hash_value, self.mapped_strings)
        self.assertEqual(result, string_hashed)
        self.assertIsNotNone(result)
        self.assertNotEqual(result, "")
        self.assertIsInstance(result, str)

        # Test for a non-existent hash value
        result = reverse_hash(12345, self.mapped_strings)
        self.assertIsNone(result)

    def test_reverse_hash_sensitive_info(self):
        # Additional test cases for string_hash2
        string_hash2 = "SensitiveInformationIsHere"
        hash_value_2 = string_hash(string_hash2)

        self.mapped_strings[hash_value_2] = string_hash2

        result_2 = reverse_hash(hash_value_2, self.mapped_strings)

        self.assertEqual(result_2, string_hash2)
        self.assertIsNotNone(result_2)
        self.assertNotEqual(result_2, "")
        self.assertIsInstance(result_2, str)

    def test_reverse_hash_special_characters(self):
        # Additional test cases for string_hash3
        string_hash3 = "What if there are numbers (333!) and symbols too?"
        hash_value_3 = string_hash(string_hash3)

        self.mapped_strings[hash_value_3] = string_hash3

        result_3 = reverse_hash(hash_value_3, self.mapped_strings)

        self.assertEqual(result_3, string_hash3)
        self.assertIsNotNone(result_3)
        self.assertNotEqual(result_3, "")
        self.assertIsInstance(result_3, str)
        
    def tearDown(self):
        pass
        
    @classmethod
    def tearDownClass(cls):
        cls.mapped_strings = None
        
class TestCheckCredentials(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.df = pd.DataFrame({
            'username': [str(string_hash('username1'))],
            'password': [str(string_hash('password1'))]
        })
        cls.df2 = pd.DataFrame({
            'username': [str(string_hash('222222222'))],
            'password': [str(string_hash('222222222'))]
        })
        cls.df3 = pd.DataFrame({
            'username': [str(string_hash('username3!'))],
            'password': [str(string_hash('password3!'))]
        })
        cls.df4 = pd.DataFrame({
            'username': [str(string_hash('USERNAME4'))],
            'password': [str(string_hash('PASSWORD4'))]
        })

        cls.result_true1 = check_credentials('username1', 'password1', cls.df)
        cls.result_true2 = check_credentials('222222222', '222222222', cls.df2)
        cls.result_true3 = check_credentials('username3!', 'password3!', cls.df3)
        cls.result_true4 = check_credentials('USERNAME4', 'PASSWORD4', cls.df4)

        cls.result_false1 = check_credentials('Unknown1', 'Password1', cls.df)
        cls.result_false2 = check_credentials('333333333', '333333333', cls.df2)
        cls.result_false3 = check_credentials('Unknown3!', 'Password3!', cls.df3)
        cls.result_false4 = check_credentials('UNKNOWN', 'UNKNOWN', cls.df4)

        cls.result_none1 = check_credentials('username1', 'password1', None)
        cls.result_none2 = check_credentials('222222222', '222222222', None)
        cls.result_none3 = check_credentials('username3!', 'password3!', None)
        cls.result_none4 = check_credentials('USERNAME4', 'PASSWORD4', None)

    def setUp(self):
        pass

    def test_check_credentials_match(self):
        # Test for matching credentials in DataFrame
        self.assertTrue(self.result_true1)
        self.assertTrue(self.result_true2)
        self.assertTrue(self.result_true3)
        self.assertTrue(self.result_true4)

    def test_check_credentials_no_match(self):
        # Test for no matching credentials in DataFrame
        self.assertFalse(self.result_false1)
        self.assertFalse(self.result_false2)
        self.assertFalse(self.result_false3)
        self.assertFalse(self.result_false4)

    def test_check_credentials_none_df(self):
        # Test when df is None
        self.assertFalse(self.result_none1)
        self.assertFalse(self.result_none2)
        self.assertFalse(self.result_none3)
        self.assertFalse(self.result_none4)
        
    def tearDown(self):
        pass
        
    @classmethod
    def tearDownClass(cls):
        cls.df = None
        cls.df2 = None
        cls.df3 = None
        cls.df4 = None

if __name__ == '__main__':
    unittest.main(argv=[''], verbosity=2, exit=False)


